import { UserEntity } from '@modules/user/core/domain/user.entity';
import { CreateUserProps } from '@modules/user/core/domain/user.types';
import { PrismaService } from '@modules/user/infrastructure/prisma.service';
import { UserRepository } from '@modules/user/infrastructure/user.repository';
import {
  USER_MESSAGE_PUBLISHER,
  USER_REPOSITORY,
} from '@modules/user/user.di-tokens';
import { UserMapper } from '@modules/user/user.mapper';
import { ConfigModule } from '@nestjs/config';
import { EventEmitterModule } from '@nestjs/event-emitter';
import { Test } from '@nestjs/testing';

describe('User Repository', () => {
  let prismaService: PrismaService;
  let userRepository: UserRepository;

  const executeInsertCommand = async (table: string, object: any) => {
    const command = `INSERT INTO "${table}" ("${Object.keys(object).join(
      '","',
    )}") VALUES ('${Object.values(object).join("','")}')`;
    await prismaService.$executeRawUnsafe(command);
  };
  const getSeed = (index: number, uuid: string): string => {
    return `${uuid.slice(0, -2)}${index.toString(16).padStart(2, '0')}`;
  };

  const baseUuid = {
    uuid: 'be459a29-7a41-4c0b-b371-abe90bfb6f00',
  };

  const createUsers = async (nbToCreate = 10) => {
    for (let i = 0; i < nbToCreate; i++) {
      const userToCreate = {
        uuid: getSeed(i, baseUuid.uuid),
        firstName: `John${i}`,
        lastName: `Doe${i}`,
        email: `john.doe${i}@email.com`,
        phone: `+33611223344${i}`,
        createdAt: '2023-07-24 13:07:05.000',
        updatedAt: '2023-07-24 13:07:05.000',
      };
      userToCreate.uuid = getSeed(i, baseUuid.uuid);
      await executeInsertCommand('user', userToCreate);
    }
  };

  const mockMessagePublisher = {
    publish: jest.fn().mockImplementation(),
  };

  const mockLogger = {
    log: jest.fn(),
    warn: jest.fn(),
    error: jest.fn(),
  };

  beforeAll(async () => {
    const module = await Test.createTestingModule({
      imports: [
        EventEmitterModule.forRoot(),
        ConfigModule.forRoot({ isGlobal: true }),
      ],
      providers: [
        PrismaService,
        UserMapper,
        {
          provide: USER_REPOSITORY,
          useClass: UserRepository,
        },
        {
          provide: USER_MESSAGE_PUBLISHER,
          useValue: mockMessagePublisher,
        },
      ],
    })
      // disable logging
      .setLogger(mockLogger)
      .compile();

    prismaService = module.get<PrismaService>(PrismaService);
    userRepository = module.get<UserRepository>(USER_REPOSITORY);
  });

  afterAll(async () => {
    await prismaService.$disconnect();
  });

  beforeEach(async () => {
    await prismaService.user.deleteMany();
  });

  describe('findOneById', () => {
    it('should return a user', async () => {
      await createUsers(1);
      const result = await userRepository.findOneById(baseUuid.uuid);
      expect(result.id).toBe(baseUuid.uuid);
    });
  });

  describe('create', () => {
    it('should create a user', async () => {
      const beforeCount = await prismaService.user.count();

      const createUserProps: CreateUserProps = {
        firstName: 'Jane',
        lastName: 'Doe',
        email: 'jane.doe@email.com',
        phone: '+33622334455',
      };

      const userToCreate: UserEntity = UserEntity.create(createUserProps);
      await userRepository.insert(userToCreate);

      const afterCount = await prismaService.user.count();

      expect(afterCount - beforeCount).toBe(1);
    });
  });
});
