import { Mapper } from '@mobicoop/ddd-library';
import { Injectable } from '@nestjs/common';
import { UserEntity } from './core/domain/user.entity';
import {
  UserReadModel,
  UserWriteModel,
} from './infrastructure/user.repository';
import { UserResponseDto } from './interface/dtos/user.response.dto';

/**
 * Mapper constructs objects that are used in different layers:
 * Record is an object that is stored in a database,
 * Entity is an object that is used in application domain layer,
 * and a ResponseDTO is an object returned to a user (usually as json).
 */

@Injectable()
export class UserMapper
  implements Mapper<UserEntity, UserReadModel, UserWriteModel, UserResponseDto>
{
  toPersistence = (entity: UserEntity): UserWriteModel => {
    const copy = entity.getProps();
    const record: UserWriteModel = {
      uuid: entity.id,
      firstName: copy.firstName,
      lastName: copy.lastName,
      email: copy.email,
      phone: copy.phone,
    };
    return record;
  };

  toDomain = (record: UserReadModel): UserEntity => {
    const entity = new UserEntity({
      id: record.uuid,
      createdAt: new Date(record.createdAt),
      updatedAt: new Date(record.updatedAt),
      props: {
        firstName: record.firstName,
        lastName: record.lastName,
        email: record.email,
        phone: record.phone,
      },
    });
    return entity;
  };

  toResponse = (entity: UserEntity): UserResponseDto => {
    const props = entity.getProps();
    const response = new UserResponseDto(entity);
    response.firstName = props.firstName;
    response.lastName = props.lastName;
    response.email = props.email;
    response.phone = props.phone;
    return response;
  };
}
